      subroutine verttransform(n,uuh,vvh,wwh)
C                              i  i   i   i   i
********************************************************************************
* $Id: verttransform.f,v 1.1 2006/08/10 15:30:51 paul Exp paul $
* $Log: verttransform.f,v $
* Revision 1.1  2006/08/10 15:30:51  paul
* Initial revision
* Revision 1.2  20080705 P. Seibert
* - Replace grid point (1,1) by the lowest one for level construction
*   as in other newer Flexpart versions
* - Get rid of doubling of layers; 
* - correct the w correction scheme which was wrong in all Flexpart-MM5 
*   versions up to now.
* 
*                                                                              *
*     This subroutine transforms temperature, dew point temperature and        *
*     wind components from sigma to Flexpart-z levels.                     
*     The vertical wind component is in m/s, but needs to be corrected for
*     the terrain-following z coordinates in Flexpart                           
*     In addition, this routine calculates vertical density gradients          *
*     needed for the parameterization of the turbulent velocities.             *
*                                                                              *
*     Author: A. Stohl, G. Wotawa                                              *
*                                                                              *
*     12 August 1996                                                           *
*     Update: 16 January 1998                                                  *
*                                                                              *
*     Major update: 17 February 1999                                           *
*     by G. Wotawa                                                             *
*                                                                              *
*     - Vertical levels for u, v and w are put together                        *
*     - Slope correction for vertical velocity: Modification of calculation    *
*       procedure                                                              *
*                                                                              *
********************************************************************************
*  Changes, Bernd C. Krueger, Feb. 2001:
*        Variables tth and qvh (on eta coordinates) from common block
********************************************************************************
*                                                                              *
* Variables:                                                                   *
* nx,ny,nz                        field dimensions in x,y and z direction      *
* uu(0:nxmax,0:nymax,nzmax,2)     wind components in x-direction [m/s]         *
* vv(0:nxmax,0:nymax,nzmax,2)     wind components in y-direction [m/s]         *
* ww(0:nxmax,0:nymax,nzmax,2)     wind components in z-direction [deltaeta/s]  *
* tt(0:nxmax,0:nymax,nzmax,2)     temperature [K]                              *
* pv(0:nxmax,0:nymax,nzmax,2)     potential voriticity (pvu)                   *
* ps(0:nxmax,0:nymax,2)           surface pressure [Pa]                        *
*                                                                              *
********************************************************************************

      include 'includepar'
      include 'includecom'

      integer ix,jy,kz,iz,n,kmin,kl,klp,ix1,jy1,ixp,jyp,ixm,jym
      real uvzlev(nuvzmax),rhoh(nuvzmax),pinmconv(nzmax)
      real ew,pint,tv,tvold,pold,const,dz1,dz2,dz,ui,vi
      real xlon,ylat,xlonr,dzdx,dzdy
      real dzdx1,dzdx2,dzdy1,dzdy2
      real uuaux,vvaux,uupolaux,vvpolaux,ddpol,ffpol,wdummy
      real uuh(0:nxmax-1,0:nymax-1,nuvzmax)
      real vvh(0:nxmax-1,0:nymax-1,nuvzmax)
      real pvh(0:nxmax-1,0:nymax-1,nuvzmax)
      real wwh(0:nxmax-1,0:nymax-1,nuvzmax) !nwzmax
      real wzlev(nwzmax),uvwzlev(0:nxmax-1,0:nymax-1,nzmax),pih
      parameter(const=r_air/ga,pih=pi/180.)
      real geopot1,sigmatoh
!      real tkeh(0:nxmax-1,0:nymax-1,nuvzmax) !IN COMMON
      real wm,dxtmp,dytmp,psmax
      logical init
      save init
      data init/.true./


**************************************************************************
* If verttransform is called the first time, initialize heights of the   *
* z levels in meter. The heights are the heights of model levels, where  *
* u,v,T and qv are given, and of the interfaces, where w is given. So,   *
* the vertical resolution in the z system is doubled. As reference point,*
* the lower left corner of the grid is used.                             *
* Unlike in the eta system, no difference between heights for u,v and    *
* heights for w exists.                                                  *
**************************************************************************

      if (init) then

C Search for a point with high surface pressure (i.e. not above significant topography)
C Then, use this point to construct a reference z profile, to be used at all times
***************************************************************************************

	  psmax=0.
        do 4 jy=0,nymin1
          do 4 ix=0,nxmin1
            if (ps(ix,jy,1,n).gt.psmax) then
		  psmax = ps(ix,jy,1,n)
              ixm=ix
              jym=jy
            endif
4       continue

        height(1)=0.

        geopot1=sigmatoh(sigmafull(1),psmax,ptop,p00,ts0,ta)

        do 7,kz=2,nz
           height(kz)=sigmatoh(sigmanew(kz),psmax,ptop,p00,ts0,ta)
     & 	          -geopot1
 7      continue
 
C Determine highest levels that can be within PBL
*************************************************

        do 8 kz=1,nz
          if (height(kz).gt.hmixmax) then
            nmixz=kz
            goto 9
          endif
8         continue
9       continue

C Do not repeat initialization of the Cartesian z grid
******************************************************

        init=.false.

      endif


C Loop over the whole grid
**************************

      do 10 jy=0,nymin1
        do 10 ix=0,nxmin1
           geopot1=sigmatoh(sigmafull(1),ps(ix,jy,1,n),ptop,p00,ts0,ta)
C          tvold=tt2(ix,jy,1,n)*(1.+0.378*ew(td2(ix,jy,1,n))/
C     &                                   ps(ix,jy,1,n))
C          pold=ps(ix,jy,1,n)
          uvzlev(1)=0.
          wzlev(1)=0.
          wm=qvh(ix,jy,1,n)
          tv=tth(ix,jy,1,n)*(1.+.378+wm/(wm+.622))
          rhoh(1)=ps(ix,jy,1,n)/(r_air*tv)


C Compute heights of sigma levels
*******************************

          do 20 kz=2,nuvz
            pint=sigmahalf(kz)*(ps(ix,jy,1,n)-ptop)+ptop !Ref. pressure level
            wm=qvh(ix,jy,kz,n)/(1.-qvh(ix,jy,kz,n))
            tv=tth(ix,jy,kz,n)*(1.+0.378*wm/(wm+.622))
            rhoh(kz)=pint/(r_air*tv)
C            if (abs(tv-tvold).gt.0.2) then
            uvzlev(kz)=sigmatoh(sigmahalf(kz),ps(ix,jy,1,n),ptop,p00,
     +                          ts0,ta)-geopot1
 20       continue

          do 21 kz=2,nwz-1
            wzlev(kz)=sigmatoh(sigmafull(kz),ps(ix,jy,1,n),ptop,p00,
     +                         ts0,ta)-geopot1
 21       continue

          uvwzlev(ix,jy,1)=0.0
          do 22 kz=2,nuvz
22          uvwzlev(ix,jy,kz)=uvzlev(kz)

C Levels where u,v,t and q are given
*************************************
          uu(ix,jy,1,n)=uuh(ix,jy,1)
          vv(ix,jy,1,n)=vvh(ix,jy,1)
          tt(ix,jy,1,n)=tth(ix,jy,1,n)
          qv(ix,jy,1,n)=qvh(ix,jy,1,n)
C          pv(ix,jy,1,n)=pvh(ix,jy,1)
          rho(ix,jy,1,n)=rhoh(1)
          tke(ix,jy,1,n)=tkeh(ix,jy,1)

          uu(ix,jy,nz,n)=uuh(ix,jy,nuvz)
          vv(ix,jy,nz,n)=vvh(ix,jy,nuvz)
          tt(ix,jy,nz,n)=tth(ix,jy,nuvz,n)
          qv(ix,jy,nz,n)=qvh(ix,jy,nuvz,n)
C          pv(ix,jy,nz,n)=pvh(ix,jy,nuvz)
          rho(ix,jy,nz,n)=rhoh(nuvz)
          tke(ix,jy,nz,n)=tkeh(ix,jy,nuvz)
          kmin=2
          do 30 iz=2,nz-1
            do 35 kz=kmin,nuvz
              if (height(iz).gt.uvzlev(nuvz)) then
                uu(ix,jy,iz,n)=uu(ix,jy,nz,n)
                vv(ix,jy,iz,n)=vv(ix,jy,nz,n)
                tt(ix,jy,iz,n)=tt(ix,jy,nz,n)
                qv(ix,jy,iz,n)=qv(ix,jy,nz,n)
C                pv(ix,jy,iz,n)=pv(ix,jy,nz,n)
                tke(ix,jy,iz,n)=tke(ix,jy,nz,n)
                rho(ix,jy,iz,n)=rho(ix,jy,nz,n)
                goto 30
              endif
              if ((height(iz).gt.uvzlev(kz-1)).and.
     +            (height(iz).le.uvzlev(kz))) then
               dz1=height(iz)-uvzlev(kz-1)
               dz2=uvzlev(kz)-height(iz)
               dz=dz1+dz2
               uu(ix,jy,iz,n)=(uuh(ix,jy,kz-1)*dz2+uuh(ix,jy,kz)*dz1)/dz
               vv(ix,jy,iz,n)=(vvh(ix,jy,kz-1)*dz2+vvh(ix,jy,kz)*dz1)/dz
               tt(ix,jy,iz,n)=(tth(ix,jy,kz-1,n)*dz2
     $              +tth(ix,jy,kz,n)*dz1)/dz
               qv(ix,jy,iz,n)=(qvh(ix,jy,kz-1,n)*dz2
     $              +qvh(ix,jy,kz,n)*dz1)/dz
C               pv(ix,jy,iz,n)=(pvh(ix,jy,kz-1)*dz2+pvh(ix,jy,kz)*dz1)/dz
               tke(ix,jy,iz,n)=(tkeh(ix,jy,kz-1)*dz2
     +              +tkeh(ix,jy,kz)*dz1)/dz
CPSK FIXXME rho ?
               rho(ix,jy,iz,n)=(rhoh(kz-1)*dz2+rhoh(kz)*dz1)/dz
               kmin=kz
!               print*,uu(ix,jy,iz,n),vv(ix,jy,iz,n),ix,jy,iz,n
               goto 30
              endif
35            continue
30          continue

C Levels where w is given
**************************

          ww(ix,jy,1,n)=wwh(ix,jy,1)
          ww(ix,jy,nz,n)=wwh(ix,jy,nwz)
          kmin=2
          do 40 iz=2,nz
            do 45 kz=kmin,nwz
              if ((height(iz).gt.wzlev(kz-1)).and.
     +            (height(iz).le.wzlev(kz))) then
               dz1=height(iz)-wzlev(kz-1)
               dz2=wzlev(kz)-height(iz)
               dz=dz1+dz2
               ww(ix,jy,iz,n)=(wwh(ix,jy,kz-1)*dz2+wwh(ix,jy,kz)*dz1)/dz
               kmin=kz
               goto 40
              endif
45            continue
40          continue

C Compute density gradients at intermediate levels
**************************************************

          drhodz(ix,jy,1,n)=(rho(ix,jy,2,n)-rho(ix,jy,1,n))/
     +      (height(2)-height(1))
          do 50 kz=2,nz-1
50          drhodz(ix,jy,kz,n)=(rho(ix,jy,kz+1,n)-rho(ix,jy,kz-1,n))/
     +      (height(kz+1)-height(kz-1))
          drhodz(ix,jy,nz,n)=drhodz(ix,jy,nz-1,n)

10        continue

*****************************************************************
C Compute slope of Flexpart levels in windward direction and resulting
C vertical wind correction
*****************************************************************

      do 60 jy=1,ny-2
        do 60 ix=1,nx-2

          ix1=ix-1
          jy1=jy-1
          ixp=ix+1
          jyp=jy+1

CPS getting the slope of the Flexpart z levels is trivial, as they
CPS are parallel to the model orography
          dzdx=(oro(ixp,jy)-oro(ix1,jy))/2.
          dzdy=(oro(ix,jyp)-oro(ix,jy1))/2.

          do 41 iz=1,nz
CPSK FIXXME / by griddistance in m dx=dy in MM5!!!
CPS dxconst=dyconst= 1./dx where dx is the grid distance in metres
CPS at true latitude. Should be replaced by exact distances.
            ui=uu(ix,jy,iz,n)*dxconst
            vi=vv(ix,jy,iz,n)*dyconst

CPS ww as it comes from MM5 is ordinary vertical velocity (individual
CPS change of height above sea level. In Flexpart we need vertical velocity
CPS in m/s but relative to model topography. So if a particle moves upslope
CPS parallel to the ground, it should have ww=0. Its velocity in MM5 will be
CPS dzdx*ui + dzdy*vi (positive if oro increases along the wind direction)
CPS Thus we have to subtract this term.

              ww(ix,jy,iz,n) = ww(ix,jy,iz,n) - (dzdx*ui + dzdy*vi)
 
41          continue
60        continue

C If north pole is in the domain, calculate wind velocities in polar
C stereographic coordinates
********************************************************************
 
      if (nglobal) then
         print '(" *** STOP in verttransform    ***")'
         print '(" *** the NGLOBAL code section ***")'
         print '(" *** should NOT be active !!! ***")'
         stop
C
C        do 74 jy=int(switchnorthg)-2,nymin1
C          ylat=ylat0+float(jy)*dy
C          do 74 ix=0,nxmin1
C            xlon=xlon0+float(ix)*dx
C            do 74 iz=1,nz
C74            call cc2gll(northpolemap,ylat,xlon,uu(ix,jy,iz,n),
C     +        vv(ix,jy,iz,n),uupol(ix,jy,iz,n),
C     +        vvpol(ix,jy,iz,n))
C
C
C        do 76 iz=1,nz
C
C* CALCULATE FFPOL, DDPOL FOR CENTRAL GRID POINT
C          xlon=xlon0+float(nx/2-1)*dx
C          xlonr=xlon*pi/180.
C          ffpol=sqrt(uu(nx/2-1,nymin1,iz,n)**2+
C     &               vv(nx/2-1,nymin1,iz,n)**2)
C          if(vv(nx/2-1,nymin1,iz,n).lt.0.) then
C            ddpol=atan(uu(nx/2-1,nymin1,iz,n)/
C     &                 vv(nx/2-1,nymin1,iz,n))-xlonr
C          else
C            ddpol=pi+atan(uu(nx/2-1,nymin1,iz,n)/
C     &                    vv(nx/2-1,nymin1,iz,n))-xlonr
C          endif
C          if(ddpol.lt.0.) ddpol=2.0*pi+ddpol
C          if(ddpol.gt.2.0*pi) ddpol=ddpol-2.0*pi
C 
C* CALCULATE U,V FOR 180 DEG, TRANSFORM TO POLAR STEREOGRAPHIC GRID
C          xlon=180.0
C          xlonr=xlon*pi/180.
C          ylat=90.0
C          uuaux=-ffpol*sin(xlonr+ddpol)
C          vvaux=-ffpol*cos(xlonr+ddpol)
C          call cc2gll(northpolemap,ylat,xlon,uuaux,vvaux,uupolaux,
C     +      vvpolaux)
C      
C          jy=nymin1
C          do 76 ix=0,nxmin1
C            uupol(ix,jy,iz,n)=uupolaux
C            vvpol(ix,jy,iz,n)=vvpolaux
C76      continue
C 
C 
C* Fix: Set W at pole to the zonally averaged W of the next equator-
C* ward parallel of latitude
C 
C      do 85 iz=1,nz
C          wdummy=0.
C          jy=ny-2
C          do 80 ix=0,nxmin1
C80          wdummy=wdummy+ww(ix,jy,iz,n)
C          wdummy=wdummy/float(nx)
C          jy=nymin1
C          do 85 ix=0,nxmin1
C85          ww(ix,jy,iz,n)=wdummy
C 
      endif 

 
C If south pole is in the domain, calculate wind velocities in polar
C stereographic coordinates
********************************************************************
C 
      if (sglobal) then
         print '(" *** STOP in verttransform    ***")'
         print '(" *** the SGLOBAL code section ***")'
         print '(" *** should NOT be active !!! ***")'
         stop
C
C        do 77 jy=0,int(switchsouthg)+3
C          ylat=ylat0+float(jy)*dy
C          do 77 ix=0,nxmin1
C            xlon=xlon0+float(ix)*dx
C            do 77 iz=1,nz
C77            call cc2gll(southpolemap,ylat,xlon,uu(ix,jy,iz,n),
C     +        vv(ix,jy,iz,n),uupol(ix,jy,iz,n),
C     +        vvpol(ix,jy,iz,n))
C      
C        do 79 iz=1,nz
C 
C* CALCULATE FFPOL, DDPOL FOR CENTRAL GRID POINT
C          xlon=xlon0+float(nx/2-1)*dx
C          xlonr=xlon*pi/180.
C          ffpol=sqrt(uu(nx/2-1,0,iz,n)**2+
C     &               vv(nx/2-1,0,iz,n)**2)
C          if(vv(nx/2-1,0,iz,n).lt.0.) then
C            ddpol=atan(uu(nx/2-1,0,iz,n)/
C     &                 vv(nx/2-1,0,iz,n))+xlonr
C          else
C            ddpol=pi+atan(uu(nx/2-1,0,iz,n)/
C     &                    vv(nx/2-1,0,iz,n))+xlonr
C          endif
C          if(ddpol.lt.0.) ddpol=2.0*pi+ddpol
C          if(ddpol.gt.2.0*pi) ddpol=ddpol-2.0*pi
C 
C* CALCULATE U,V FOR 180 DEG, TRANSFORM TO POLAR STEREOGRAPHIC GRID
C          xlon=180.0
C          xlonr=xlon*pi/180.
C          ylat=-90.0
C          uuaux=+ffpol*sin(xlonr-ddpol)
C          vvaux=-ffpol*cos(xlonr-ddpol)
C          call cc2gll(northpolemap,ylat,xlon,uuaux,vvaux,uupolaux,
C     +      vvpolaux)
C      
C          jy=0
C          do 79 ix=0,nxmin1
C            uupol(ix,jy,iz,n)=uupolaux
C79          vvpol(ix,jy,iz,n)=vvpolaux
C 
C 
C* Fix: Set W at pole to the zonally averaged W of the next equator-
C* ward parallel of latitude
C 
C        do 95 iz=1,nz
C          wdummy=0.
C          jy=1
C          do 90 ix=0,nxmin1
C90          wdummy=wdummy+ww(ix,jy,iz,n)
C          wdummy=wdummy/float(nx)
C          jy=0
C          do 95 ix=0,nxmin1
C95          ww(ix,jy,iz,n)=wdummy
      endif



      end
